@@ -742,6 +742,9 @@ unsigned int g_supplicant_interface_get_max_scan_ssids(
 	if (interface == NULL)
 		return 0;
 
+	if (interface->max_scan_ssids == 0)
+		return WPAS_MAX_SCAN_SSIDS;
+
 	return interface->max_scan_ssids;
 }
 
@@ -2376,7 +2379,7 @@ static void interface_create_result(const char *error,
 	SUPPLICANT_DBG("");
 
 	if (error != NULL) {
-		g_critical("error %s", error);
+		g_warning("error %s", error);
 		err = -EIO;
 		goto done;
 	}
@@ -2449,7 +2452,6 @@ static void interface_get_result(const char *error,
 
 	if (error != NULL) {
 		SUPPLICANT_DBG("Interface not created yet");
-		err = -EIO;
 		goto create;
 	}
 
@@ -2618,7 +2620,7 @@ static void interface_scan_result(const char *error,
 	}
 
 	if (data != NULL && data->scan_params != NULL)
-		g_free(data->scan_params);
+		g_supplicant_free_scan_params(data->scan_params);
 
 	dbus_free(data);
 }
@@ -2643,7 +2645,7 @@ static void add_scan_frequencies(DBusMessageIter *iter,
 	unsigned int freq;
 	int i;
 
-	for (i = 0; i < G_SUPPLICANT_MAX_FAST_SCAN; i++) {
+	for (i = 0; i < scan_data->num_ssids; i++) {
 		freq = scan_data->freqs[i];
 		if (!freq)
 			break;
@@ -2668,11 +2670,13 @@ static void append_ssid(DBusMessageIter *iter,
 static void append_ssids(DBusMessageIter *iter, void *user_data)
 {
 	GSupplicantScanParams *scan_data = user_data;
-	int i;
+	GSList *list;
 
-	for (i = 0; i < scan_data->num_ssids; i++)
-		append_ssid(iter, scan_data->ssids[i].ssid,
-					scan_data->ssids[i].ssid_len);
+	for (list = scan_data->ssids; list; list = list->next) {
+		struct scan_ssid *scan_ssid = list->data;
+
+		append_ssid(iter, scan_ssid->ssid, scan_ssid->ssid_len);
+	}
 }
 
 static void supplicant_add_scan_frequency(DBusMessageIter *dict,
@@ -2683,7 +2687,7 @@ static void supplicant_add_scan_frequency(DBusMessageIter *dict,
 	DBusMessageIter entry, value, array;
 	const char *key = "Channels";
 
-	if (scan_params->freqs[0] != 0) {
+	if (scan_params->freqs && scan_params->freqs[0] != 0) {
 		dbus_message_iter_open_container(dict, DBUS_TYPE_DICT_ENTRY,
 						NULL, &entry);
 
@@ -2957,18 +2961,49 @@ static dbus_bool_t is_psk_raw_key(const char *psk)
 	return TRUE;
 }
 
+static unsigned char hexchar2bin(char c)
+{
+	if ((c >= '0') && (c <= '9'))
+		return c - '0';
+	else if ((c >= 'A') && (c <= 'F'))
+		return c - 'A' + 10;
+	else if ((c >= 'a') && (c <= 'f'))
+		return c - 'a' + 10;
+	else
+		return c;
+}
+
+static void hexstring2bin(const char *string, unsigned char *data, size_t data_len)
+{
+	size_t i;
+
+	for (i = 0; i < data_len; i++)
+		data[i] = (hexchar2bin(string[i * 2 + 0]) << 4 |
+			   hexchar2bin(string[i * 2 + 1]) << 0);
+}
+
 static void add_network_security_psk(DBusMessageIter *dict,
 					GSupplicantSSID *ssid)
 {
 	if (ssid->passphrase && strlen(ssid->passphrase) > 0) {
+		const char *key = "psk";
 
-		if (is_psk_raw_key(ssid->passphrase) == TRUE)
-			supplicant_dbus_property_append_fixed_array(dict,
-							"psk", DBUS_TYPE_BYTE,
-							&ssid->passphrase, 64);
-		else
-			supplicant_dbus_dict_append_basic(dict, "psk",
-							DBUS_TYPE_STRING,
+		if (is_psk_raw_key(ssid->passphrase) == TRUE) {
+			unsigned char data[32];
+			unsigned char *datap = data;
+
+			/* The above pointer alias is required by D-Bus because
+			 * with D-Bus and GCC, non-heap-allocated arrays cannot
+			 * be passed directly by their base pointer. */
+
+			hexstring2bin(ssid->passphrase, datap, sizeof(data));
+
+			supplicant_dbus_dict_append_fixed_array(dict,
+							key, DBUS_TYPE_BYTE,
+							&datap, sizeof(data));
+		} else
+			supplicant_dbus_dict_append_basic(dict,
+							key, DBUS_TYPE_STRING,
 							&ssid->passphrase);
 	}
 }
@@ -3441,8 +3476,10 @@ static void interface_disconnect_result(const char *error,
 	/* If we are disconnecting from previous WPS successful
 	 * association. i.e.: it did not went through AddNetwork,
 	 * and interface->network_path was never set. */
-	if (data->interface->network_path == NULL)
+	if (data->interface->network_path == NULL) {
+		dbus_free(data);
 		return;
+	}
 
 	network_remove(data);
 }
